@@ -1,18 +1,22 @@
-package Devel::Examine::Subs 1.43;
+package Devel::Examine::Subs;
 use 5.012;
 use warnings;
 use strict;
 
+our $VERSION = '1.45';
+
 use Carp;
 use Data::Compare;
 use Data::Dumper;
 use Devel::Examine::Subs::Engine;
 use Devel::Examine::Subs::Preprocessor;
 use Devel::Examine::Subs::Prefilter;
+use File::Basename;
 use File::Copy;
+use File::Edit::Portable;
 use File::Find;
 use PPI;
-use Symbol;
+use Symbol qw(delete_package);
 
 BEGIN {
 
@@ -37,7 +41,8 @@ sub new {
    
     # set up for tracing
 
-    if ($ENV{DTS_ENABLE} && $ENV{DES_TRACE}){
+    if ($ENV{DES_TRACE}){
+        $ENV{DTS_ENABLE} = 1;
         $ENV{TRACE} = 1;
     }
 
@@ -56,31 +61,16 @@ sub new {
 
     return $self;
 }
-sub run {
-
+sub all {
+    
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my $p = shift;
+    my $p = $self->_params(@_);
 
-    $self->_config($p);
-
-    $self->_run_end(0);
-
-    my $struct;
-
-    if ($self->{params}{directory}){
-        $struct = $self->_run_directory;
-    }
-    else {
-        $struct = $self->_core;
-        $self->_write_file if $self->{write_file_contents};
-    }
-
-    $self->_run_end(1);
-    $self->_clean_core_config;
-
-    return $struct;
+    $self->{params}{engine} = 'all';
+    
+    $self->run($p);
 }
 sub has {
     
@@ -105,17 +95,6 @@ sub missing {
     
     $self->run($p);
 }
-sub all {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{engine} = 'all';
-    
-    $self->run($p);
-}
 sub lines {
     
     trace() if $ENV{TRACE};
@@ -136,7 +115,19 @@ sub module {
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my $p = $self->_params(@_);
+
+    my $p;
+
+    # allow for single string value
+
+    if (@_ == 1){
+        my %p;
+        $p{module} = shift;
+        $p = $self->_params(%p);
+    }
+    else {
+        $p = $self->_params(@_);
+    }
 
     # set the preprocessor up, and have it return before
     # the building/compiling of file data happens
@@ -174,6 +165,18 @@ sub search_replace {
 
     $self->run($p);
 }
+sub replace {
+
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    $self->{params}{pre_proc} = 'replace';
+    $self->{params}{pre_proc_return} = 1;
+
+    $self->run($p);
+}
 sub inject_after {
     
     trace() if $ENV{TRACE};
@@ -207,18 +210,6 @@ sub inject {
 
     $self->run($p);
 }
-sub replace {
-
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{pre_proc} = 'replace';
-    $self->{params}{pre_proc_return} = 1;
-
-    $self->run($p);
-}
 sub remove {
     
     trace() if $ENV{TRACE};
@@ -233,7 +224,7 @@ sub remove {
 }
 
 #
-# publicly available private methods
+# publicly available semi-private developer methods
 #
 
 sub add_functionality {
@@ -297,6 +288,21 @@ sub add_functionality {
 
     $p->{write_file_contents} = \@code;
 }
+sub engines {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $module = $self->{namespace} . "::Engine";
+    my $engine = $module->new;
+ 
+    my @engines;
+
+    for (keys %{$engine->_dt}){
+        push @engines, $_ if $_ !~ /^_/;
+    }
+    return @engines;
+}
 sub pre_procs {
     
     trace() if $ENV{TRACE};
@@ -327,20 +333,30 @@ sub pre_filters {
     }
     return @pre_filters;
 }
-sub engines {
-    
+sub run {
+
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my $module = $self->{namespace} . "::Engine";
-    my $engine = $module->new;
- 
-    my @engines;
+    my $p = shift;
 
-    for (keys %{$engine->_dt}){
-        push @engines, $_ if $_ !~ /^_/;
+    $self->_config($p);
+
+    $self->_run_end(0);
+
+    my $struct;
+
+    if ($self->{params}{directory}){
+        $struct = $self->_run_directory;
     }
-    return @engines;
+    else {
+        $struct = $self->_core;
+        $self->_write_file if $self->{write_file_contents};
+    }
+
+    $self->_run_end(1);
+
+    return $struct;
 }
 sub valid_params {
     
@@ -354,74 +370,13 @@ sub valid_params {
 # private methods
 #
 
-sub _run_directory {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = shift;
-
-    my @files;
-
-    my $dir = $self->{params}{file};
-
-    find({wanted => sub {
-            
-            trace() if $ENV{TRACE};
-
-            return if ! -f;
-
-            my @extensions = @{$self->{params}{extensions}};
-            my $exts = join('|', @extensions);
-
-            if ($_ !~ /\.(?:$exts)$/i){
-                return;
-            }
-
-            my $file = "$File::Find::name";
-
-            push @files, $file;
-          },
-            no_chdir => 1
-        }, $dir
-    );
-
-    my %struct;
-
-    for my $file (@files){
-        
-        $self->{params}{file} = $file;
-        my $data = $self->_core($p);
-
-        $self->_write_file if $self->{write_file_contents};
-
-        my $exists = 0;
-        $exists = %$data if ref($data) eq 'HASH';
-        $exists = @$data if ref($data) eq 'ARRAY';
-
-        $struct{$file} = $data if $exists;
-    }
-
-    return \%struct;
-}
-sub _run_end {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $value = shift;
-
-    $self->{run_end} = $value if defined $value;
-
-    return $self->{run_end};
-}
 sub _cache {
     
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my $file = shift if @_;
-    my $struct = shift if @_;
+    my $file = shift;
+    my $struct = shift;
 
     if ($self->{params}{cache_dump}){
 
@@ -457,13 +412,53 @@ sub _cache_safe {
 
     return $self->{cache_safe};
 }
-sub _params {
+sub _clean_config {
     
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my %params = @_;
-    return \%params;
+    my $config_vars = shift; # href of valid params
+    my $p = shift; # href of params passed in
+
+    for my $var (keys %$config_vars){
+       
+        last if ! $self->_run_end;
+
+        # skip if it's a persistent var
+
+        next if $config_vars->{$var} == 1;
+
+        delete $self->{params}{$var};
+    }
+
+    # delete non-valid params
+
+    for my $param (keys %$p){
+        if (! exists $config_vars->{$param}){
+            print "\n\nDES::_clean_config() deleting invalid param: $param\n";
+            delete $p->{$param};
+        }
+    }
+}
+sub _clean_core_config {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+
+    # clean params that we collected after _clean_config()
+
+    delete $self->{params}{file_contents};
+
+    my @core_phases = qw(
+        pre_proc
+        pre_filter
+        engine
+    );
+
+    for (@core_phases){
+        delete $self->{params}{$_};
+    }
 }
 sub _config {
     
@@ -564,61 +559,6 @@ sub _config {
         print Dumper $self->{params};
     }
 }
-sub _clean_config {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $config_vars = shift; # href of valid params
-    my $p = shift; # href of params passed in
-
-    # params that get set after the call to _config() need
-    # to be deleted manually here
-
-#    delete $p->{file_contents};
-
-    for my $var (keys %$config_vars){
-       
-        last if ! $self->_run_end;
-
-        # skip if it's a persistent var
-
-        next if $config_vars->{$var} == 1;
-
-        delete $self->{params}{$var};
-    }
-
-    # delete non-valid params
-
-    for my $param (keys %$p){
-        if (! exists $config_vars->{$param}){
-            #warn "\n\nDES::_clean_config() deleting invalid param: $param\n";
-            delete $p->{$param};
-        }
-    }
-}
-sub _clean_core_config {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-
-    # deletes core phase info after each run
-
-    # clean params that we collected after _clean_config()
-
-    delete $self->{params}{file_contents};
-
-    my @core_phases = qw(
-        pre_proc
-        pre_filter
-        engine
-    );
-
-    for (@core_phases){
-        delete $self->{params}{$_};
-    }
-}
 sub _file {
     
     trace() if $ENV{TRACE};
@@ -677,6 +617,14 @@ sub _file {
 
    return $self->{params}{file};
 }
+sub _params {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my %params = @_;
+    return \%params;
+}
 sub _read_file {
     
     trace() if $ENV{TRACE};
@@ -685,34 +633,87 @@ sub _read_file {
     my $p = shift;
 
     my $file = $p->{file};
-    my $copy = $self->{params}{copy};
 
     return if ! $file;
 
-    copy $file, "$file.bak" or croak $!;
+    my $basename = basename($file);
+    my $bak = "$basename.bak";
 
-    open my $fh, '<', $file
-      or croak "Can't open the file $file: $!";
+    copy $file, $bak
+      or croak "DES::_read_file() can't create backup copy $bak!";
 
-    my @file_contents = <$fh>;
+    $self->{rw} = File::Edit::Portable->new;
 
-    close $fh or croak $!;
-
-    $self->{file_eol} = "\n";
-
-    if ($file_contents[0] && $file_contents[0] =~ /(\R)/){
-        $self->{file_eol} = $1;
-    }
-
-    for (@file_contents){
-        s/\R//g;
-    }
+    my @file_contents
+      = $self->{rw}->read(file => $file);
 
     @{ $p->{file_contents} } = @file_contents;
-    @{ $self->{file_contents} } = @file_contents;
 
     return @file_contents;
 }
+sub _run_directory {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = shift;
+
+    my @files;
+
+    my $dir = $self->{params}{file};
+
+    find({wanted => sub {
+            
+            trace() if $ENV{TRACE};
+
+            return if ! -f;
+
+            my @extensions = @{$self->{params}{extensions}};
+            my $exts = join('|', @extensions);
+
+            if ($_ !~ /\.(?:$exts)$/i){
+                return;
+            }
+
+            my $file = $File::Find::name;
+
+            push @files, $file;
+          },
+            no_chdir => 1
+        }, $dir
+    );
+
+    my %struct;
+
+    for my $file (@files){
+        
+        $self->{params}{file} = $file;
+        my $data = $self->_core($p);
+
+        $self->_write_file if $self->{write_file_contents};
+
+        if (ref $data eq 'HASH' || ref $data eq 'ARRAY'){
+            $struct{$file} = $data;
+        }
+    }
+
+    return \%struct;
+}
+sub _run_end {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $value = shift;
+
+    $self->{run_end} = $value if defined $value;
+
+    # we clean core_config here
+
+    $self->_clean_core_config if $value;
+
+    return $self->{run_end};
+}
 sub _write_file {
 
     trace() if $ENV{TRACE};
@@ -727,14 +728,24 @@ sub _write_file {
 
     $file = $copy if $copy;
 
-    open my $wfh, '>', $file or croak $!;
+    my $write_response;
 
-    for (@$contents){
-        print $wfh "$_$self->{file_eol}";
+    eval {
+        $write_response = $self->{rw}->write(file => $file, contents => $contents);
+    };
+
+    if ($@ || ! $write_response){
+        $@ = "\nDevel::Examine::Subs::_write_file() speaking...\n\n" .
+             "File::Edit::Portable::write() returned a failure status.\n\n" .
+             $@;
+        croak $@;
     }
-
-    close $wfh or croak !$;
 }
+
+#
+# private methods for core phases
+#
+
 sub _core {
 
     trace() if $ENV{TRACE};
@@ -755,7 +766,7 @@ sub _core {
     if ($self->{params}{pre_proc}){
         my $pre_proc = $self->_pre_proc;
 
-        $data = $pre_proc->($p);
+        $data = $pre_proc->($p, $data);
 
         if ($self->{params}{pre_proc_dump}){
             print Dumper $data;
@@ -787,15 +798,11 @@ sub _core {
         $subs = $self->_cache($p->{file});
     }
     else {
-        $subs = $self->_subs;
+        $subs = $self->_proc;
     } 
-    
-    $subs = $subs // 0;
-
+  
     return if ! $subs;
 
-    $self->{data} = $subs;
-
     # write to cache
 
     if ($self->_cache_enabled && ! $self->_cache($p->{file})){
@@ -805,10 +812,9 @@ sub _core {
     # pre engine filter
 
     if ($self->{params}{pre_filter}){
-        for my $pre_filter ($self->_pre_filter){
+        for my $pre_filter ($self->_pre_filter($p, $subs)){
             $subs = $pre_filter->($p, $subs);
             $self->{write_file_contents} = $p->{write_file_contents};
-            $self->{data} = $subs;
         }
     }  
 
@@ -818,7 +824,7 @@ sub _core {
 
     # engine
 
-    my $engine = $self->_engine($p);
+    my $engine = $self->_engine($p, $subs);
 
     if ($self->{params}{engine}){
         $subs = $engine->($p, $subs);
@@ -839,81 +845,8 @@ sub _core {
         exit;
     }
 
-    $self->{data} = $subs;
-
     return $subs;
 }
-sub _subs {
-    
-    trace() if $ENV{TRACE};
-    
-    my $self = shift;
-    
-    my $file = $self->{params}{file};
-
-    return {} if ! $file;
-
-    my $PPI_doc = PPI::Document->new($file);
-    my $PPIsubs = $PPI_doc->find("PPI::Statement::Sub");
-
-    return if ! $PPIsubs;
-
-    my @file_contents = @{ $self->{params}{file_contents} };
-
-    my %subs;
-    $subs{$file} = {};
-    
-    @{$subs{$file}{contents}} = @file_contents;
-
-    for my $PPI_sub (@{$PPIsubs}){
-
-        my $include = $self->{params}{include} // [];
-        my $exclude = $self->{params}{exclude} // [];
-
-        delete $self->{params}{include} if $exclude->[0];
-
-        my $name = $PPI_sub->name;
-
-        # skip over excluded subs
-
-        next if grep {$name eq $_ } @$exclude;
-
-        if ($include->[0]){
-            next if (! grep {$name eq $_ && $_} @$include);
-        }
-
-        $subs{$file}{subs}{$name}{start} = $PPI_sub->line_number;
-        $subs{$file}{subs}{$name}{start}--;
-
-        my $lines = $PPI_sub =~ y/\n//;
-
-        $subs{$file}{subs}{$name}{end}
-          = $subs{$file}{subs}{$name}{start} + $lines;
-
-        my $count_start = $subs{$file}{subs}{$name}{start};
-        $count_start--;
-
-        my $sub_line_count
-          = $subs{$file}{subs}{$name}{end} - $count_start;
-
-        $subs{$file}{subs}{$name}{num_lines} = $sub_line_count;
-
-        my $line_num = $subs{$file}{subs}{$name}{start};
-       
-        # pull out just the subroutine from the file 
-        # array and attach it to the structure
-
-        my @sub_definition = @file_contents[
-                                    $subs{$file}{subs}{$name}{start}
-                                    ..
-                                    $subs{$file}{subs}{$name}{end}
-                                   ];
-
-          $subs{$file}{subs}{$name}{contents} = \@sub_definition;
-    }
-   
-    return \%subs;
-}
 sub _pre_proc {
     
     trace() if $ENV{TRACE};
@@ -966,6 +899,85 @@ sub _pre_proc {
     
     return $cref;
 }
+sub _proc {
+   
+    # this method is the core data collection/manipulation
+    # routine (aka the 'Processor phase') for all of DES
+
+    # make sure all unit tests are successful after any change
+    # to this subroutine!
+
+    # if you want the data structure to look differently before 
+    # reaching here, use a pre_proc. If you want it different 
+    # afterwards, use a pre_filter or an engine
+
+    trace() if $ENV{TRACE};
+    
+    my $self = shift;
+    
+    my $file = $self->{params}{file};
+
+    return {} if ! $file;
+
+    my $PPI_doc = PPI::Document->new($file);
+    my $PPIsubs = $PPI_doc->find("PPI::Statement::Sub");
+
+    return if ! $PPIsubs;
+
+    my @file_contents = @{ $self->{params}{file_contents} };
+
+    my %subs;
+    $subs{$file} = {};
+    
+    @{$subs{$file}{contents}} = @file_contents;
+
+    for my $PPI_sub (@{$PPIsubs}){
+
+        my $include = $self->{params}{include} // [];
+        my $exclude = $self->{params}{exclude} // [];
+
+        delete $self->{params}{include} if $exclude->[0];
+
+        my $name = $PPI_sub->name;
+
+        # skip over excluded (or not included) subs
+
+        next if grep {$name eq $_ } @$exclude;
+
+        if ($include->[0]){
+            next if (! grep {$name eq $_ && $_} @$include);
+        }
+
+        $subs{$file}{subs}{$name}{start} = $PPI_sub->line_number;
+        $subs{$file}{subs}{$name}{start}--;
+
+        my $lines = $PPI_sub =~ y/\n//;
+
+        $subs{$file}{subs}{$name}{end}
+          = $subs{$file}{subs}{$name}{start} + $lines;
+
+        my $count_start = $subs{$file}{subs}{$name}{start};
+        $count_start--;
+
+        my $sub_line_count
+          = $subs{$file}{subs}{$name}{end} - $count_start;
+
+        $subs{$file}{subs}{$name}{num_lines} = $sub_line_count;
+
+        # pull out just the subroutine from the file 
+        # array and attach it to the structure
+
+        my @sub_definition = @file_contents[
+                                    $subs{$file}{subs}{$name}{start}
+                                    ..
+                                    $subs{$file}{subs}{$name}{end}
+                                   ];
+
+        $subs{$file}{subs}{$name}{contents} = \@sub_definition;
+    }
+   
+    return \%subs;
+}
 sub _pre_filter {
     
     trace() if $ENV{TRACE};
@@ -1031,11 +1043,10 @@ sub _pre_filter {
             }
 
             if ($pre_filter_dump && $pre_filter_dump == 1){
-                my $subs = $cref->($p, $self->{data});
+                my $subs = $cref->($p, $struct);
                 print Dumper $subs;
                 exit;
             }
-    
             push @pre_filters, $cref;
         }
     }
@@ -1093,7 +1104,7 @@ sub _engine {
     }
 
     if ($self->{params}{engine_dump}){
-        my $subs = $cref->($p, $self->{data});
+        my $subs = $cref->($p, $struct);
         print Dumper $subs;
         exit;
     }
@@ -1124,14 +1135,19 @@ Perl files and subs.
     my $des = Devel::Examine::Subs->new( file => $file );
 
 
+Get all sub names in a file
+
+    my $aref = $des->all;
+
+
 Get all the subs as objects
 
     $subs = $des->objects;
 
     for my $sub (@$subs){
         $sub->name;       # name of sub
-        $sub->start;      # number of first line in sub
-        $sub->end;        # number of last line in sub
+        $sub->start;      # first line number of sub in file
+        $sub->end;        # last line number of sub in file
         $sub->line_count; # number of lines in sub
         $sub->code;       # entire sub code from file
         $sub->lines;      # lines that match search term
@@ -1152,9 +1168,6 @@ Get the sub objects within a hash
               ...
     }
 
-Get all sub names in a file
-
-    my $aref = $des->all;
 
 
 Get all subs containing "string" in the body
@@ -1184,19 +1197,21 @@ Inject code into sub after a search term (preserves previous line's indenting)
 
 Print out all lines in all subs that contain a search term
 
-    my $subs = $des->objects;
+    my $subs = $des->lines(search => 'this');
 
-    for my $sub (@$subs){
-    
-        my $lines_with_search_term = $sub->lines;
+    for my $sub (keys %$subs){
 
-        for (@$lines_with_search_term){
-            my ($line_num, $text) = split /:/, $_, 2;
+        print "\nSub: $sub\n";
+
+        for my $line (@{ $subs->{$sub} }){
+            my ($line_num, $text) = each %$line;
             say "Line num: $line_num";
             say "Code: $text\n";
         }
     }
 
+
+
 The structures look a bit differently when 'file' is a directory.
 You need to add one more layer of extraction.
 
@@ -1217,9 +1232,9 @@ Print all subs within each Perl file under a directory
         print join('\t', @{$files->{$file}});
     }
 
-All methods can include or exclude specific subs
+Most methods can include or exclude specific subs
 
-    my $has = $des->has( include => [qw(dump private)] );
+    my $has = $des->has( include => ['dump', 'private'] );
 
     my $missing = $des->missing( exclude => ['this', 'that'] );
 
@@ -1237,9 +1252,7 @@ options.
 
 =head1 METHODS
 
-All public methods take their parameters as a hash (C<key =E<gt> value>).
-
-See the L<PARAMETERS> for the full list of params, and which ones 
+See the L<PARAMETERS> for the full list of params, and which ones
 are persistent across runs using the same object.
 
 
@@ -1250,7 +1263,7 @@ Mandatory parameters: C<file =E<gt> $filename>
 
 Instantiates a new object. If C<$filename> is a directory, we'll iterate
 through it finding all Perl files. If C<$filename> is a module name
-(eg: C<Data::Dumper>), we'll attempt to load the module, extract the file for
+(eg: C<Data::Dump>), we'll attempt to load the module, extract the file for
 the module, and load the file. CAUTION: this will be a production C<%INC> file
 so be careful.
 
@@ -1258,7 +1271,7 @@ Only specific params are guaranteed to stay persistent throughout a run on the
 same object, and are best set in C<new()>. These parameters are C<file>,
 C<extensions>, C<cache>, C<regex>, C<copy>, C<no_indent> and C<diff>.
 
-
+Note: omit this parameter if all you'll be using is the C<module()> method.
 
 
 
@@ -1292,6 +1305,19 @@ The exact opposite of has.
 
 
 
+=head2 C<lines>
+
+Mandatory parameters: C<search =E<gt> 'text'>
+
+Gathers together all line text and line number of all subs where the
+subroutine contains lines matching the search term.
+
+Returns a hash reference with the subroutine name as the key, the value being
+an array reference which contains a hash reference in the format line_number
+=E<gt> line_text.
+
+
+
 
 =head2 C<objects>
 
@@ -1309,9 +1335,10 @@ See L<SYNOPSIS> for the structure of each object.
 
 
 
-=head2 C<module>
+=head2 C<module('Module::Name')>
 
-Mandatory parameters: C<module =E<gt> 'Module::Name'>
+Mandatory parameters: C<'Module::Name'>. Note that this is one public method
+that takes its parameter in string format (as opposed to hash format).
 
 Returns an array reference containing the names of all subs found in the
 module's namespace symbol table.
@@ -1319,17 +1346,6 @@ module's namespace symbol table.
 
 
 
-=head2 C<lines>
-
-Mandatory parameters: C<search =E<gt> 'text'>
-
-Gathers together all line text and line number of all subs where the
-subroutine contains lines matching the search term.
-
-Returns a hash reference with the subroutine name as the key, the value being
-an array reference which contains a hash reference in the format line_number
-=E<gt> line_text.
-
 
 
 
@@ -1348,35 +1364,6 @@ with '.bak', but don't confuse this feature with the 'copy' parameter.
 
 
 
-
-=head2 C<inject>
-
-Parameters: C<inject_use =E<gt> ['use Module::Name', 'use Module2::Name']> or
-C<inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;']>
-
-C<inject_use> will inject the statements prior to all existing C<use>
-statements that already exist in the file(s). If none are found, will inject
-the statements right after a C<Package> statement if found.
-
-Technically, you don't have to inject a C<use> statement, but I'd advise it.
-
-C<inject_after_sub_def> will inject the lines of code within the array
-reference value immediately following all sub definitions in a file.
-Next line indenting is used, and sub definitions with their opening brace
-on a separate line than the definition itself is caught.
-
-
-
-=head2 C<remove>
-
-Parameters: C<delete =E<gt> ['string1', 'string2']>
-
-Deletes from the file(s) the entire lines that contain the search terms.
-
-This method is file based... the work happens prior to digging up subs, hence
-C<exclude>, C<include> and other sub-based parameters have no effect.
-
-
 =head2 C<replace>
 
 Parameters: C<exec =E<gt> $cref, limit =E<gt> 1>
@@ -1392,6 +1379,14 @@ Returns the number of lines changed in file mode, and an empty hashref in direct
 
 
 
+
+
+
+
+
+
+
+
 =head2 C<inject_after>
 
 Mandatory parameters: C<search =E<gt> 'this', code =E<gt> \@code>
@@ -1432,10 +1427,37 @@ details.
 
 
 
+=head2 C<inject>
+
+Parameters: C<inject_use =E<gt> ['use Module::Name', 'use Module2::Name']> or
+C<inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;']>
+
+C<inject_use> will inject the statements prior to all existing C<use>
+statements that already exist in the file(s). If none are found, will inject
+the statements right after a C<Package> statement if found.
+
+Technically, you don't have to inject a C<use> statement, but I'd advise it.
+
+C<inject_after_sub_def> will inject the lines of code within the array
+reference value immediately following all sub definitions in a file.
+Next line indenting is used, and sub definitions with their opening brace
+on a separate line than the definition itself is caught.
 
 
 
 
+=head2 C<remove>
+
+Parameters: C<delete =E<gt> ['string1', 'string2']>
+
+Deletes from the file(s) the entire lines that contain the search terms.
+
+This method is file based... the work happens prior to digging up subs, hence
+C<exclude>, C<include> and other sub-based parameters have no effect.
+
+
+
+=head1 C<DEVELOPER METHODS>
 
 =head2 C<valid_params>
 
@@ -1763,8 +1785,7 @@ L<https://github.com/stevieb9/devel-examine-subs>
 
 If C<Devel::Trace::Subs> is installed, you can configure stack tracing.
 
-In your calling script, set C<$ENV{DTS_ENABLE} = 1> and 
-C<$ENV{DES_TRACE} = 1>.
+In your calling script, set C<$ENV{DES_TRACE} = 1>.
 
 See C<perldoc Devel::Trace::Subs> for information on how to access the traces.
 
