@@ -1,8 +1,10 @@
-package Devel::Examine::Subs::Engine 1.43;
+package Devel::Examine::Subs::Engine;
 use 5.012;
 use strict;
 use warnings;
 
+our $VERSION = '1.45';
+
 use Carp;
 use Data::Dumper;
 use Devel::Examine::Subs;
@@ -54,7 +56,6 @@ sub _dt {
         _test => \&_test,
         _test_print => \&_test_print,
         _test_bad => \&_test_bad,
-
     };
 
     return $dt;
@@ -272,7 +273,11 @@ sub search_replace {
         my $file = $p->{file};
         my $exec = $p->{exec};
 
-        my @file_contents = @{ $p->{file_contents} } if $p->{file_contents};
+        my @file_contents;
+
+        if ($p->{file_contents}) {
+            @file_contents = @{ $p->{file_contents} };
+        }
 
         if (! $file){
             croak "\nDevel::Examine::Subs::Engine::search_replace " .
@@ -281,7 +286,7 @@ sub search_replace {
                   "file\n\n";
         }
 
-        if (! $exec || ref $exec ne 'CODE'){
+        if (! $exec || ref($exec) ne 'CODE'){
             croak "\nDevel::Examine::Subs::Engine::search_replace " .
                   " speaking:\n" .
                   "can't use search_replace engine without specifying" .
