@@ -1,8 +1,10 @@
-package Devel::Examine::Subs::Preprocessor 1.43;
+package Devel::Examine::Subs::Preprocessor;
 use 5.012;
 use strict;
 use warnings;
 
+our $VERSION = '1.45';
+
 use Carp;
 use Data::Dumper;
 
