@@ -1,8 +1,10 @@
-package Devel::Examine::Subs::Sub 1.43;
+package Devel::Examine::Subs::Sub;
 use 5.012;
 use strict;
 use warnings;
 
+our $VERSION = '1.45';
+
 use Data::Dumper;
 
 BEGIN {
